# 
# Copyright (C) 1997-2001 Herrin Software Development, Inc.
# All rights reserved.
#
# This file is part of Qtime
#

proc launchStopwatch {} {
    global gv_attr timerStack fx:mode_variable fx:readonly_modeval

    menubar ReadOnlyModeProc
    set t [menubar cget -tuple]
    if {${fx:mode_variable} != ${fx:readonly_modeval}} {
	return
    }
    set s [menubar cget -schema]
    set k [qddb_search $s -prunebyattr Job.JobNumber w $gv_attr(Job.JobNumber)]
    set flag 0
    foreach i [qddb_keylist get $k] {
	set nt [qddb_tuple read $s $i]
	if {$nt != ""} {
	    qddb_tuple delete $nt
	    set flag 1
	    break
	}
    }
    if {!$flag} {
	fx_dialog .error_dialog "Warning" "Job number $gv_attr(Job.JobNumber) has been deleted" error 0 Ok
	qddb_tuple refresh $t
	return
    }
    addToTimerHistory "$gv_attr(Job.JobNumber)" "$gv_attr(Job.JobName)"
    set win "stopwatch_$gv_attr(Job.JobNumber)"
    if {[winfo exists .$win]} {
        raise .$win
        update; update idletasks
        return
    }
    Stopwatch #auto
    timerStack push $gv_attr(Job.JobNumber) "$gv_attr(Job.JobName)"
}

proc launchStopwatchFromQueue {jobnum jobname} {
    global gv_attr fx:mode_variable timerStack fx_config

    set s [menubar cget -schema]
    set k [qddb_search $s -prunebyattr Job.JobNumber $jobnum]
    set tbl [qddb_rows select -format table -print Job.JobNumber -rowdescs on $k]
    qddb_keylist delete $k
    if {$tbl == ""} {
	fx_dialog .error_dialog "Warning" "Job number $jobnum has been deleted" error 0 Ok
	deleteFromTimerHistory $jobnum $jobname
	return
    }
    addToTimerHistory "$jobnum" "$jobname"
    set win "stopwatch_$jobnum"
    menubar SearchModeProc
    set maxrow [qddb_table row maxnum $tbl]
    if {$maxrow > 1} {
	fx_dialog .error_dialog "DATABASE ERROR" \
		"Job number $jobnum appears $maxrow times in your database!" \
		error 0 Ok
	return
    }
    menubar SetLastSearch $tbl
    if {[winfo exists .$win]} {
        raise .$win
        update; update idletasks
        return
    }
    menubar ChangeModeProc 0
    menubar ReadOnlyModeProc
    Stopwatch #auto
    timerStack push $gv_attr(Job.JobNumber) "$gv_attr(Job.JobName)"
}

proc showStopwatch {num} {
    set win "stopwatch_$num"
    if {[winfo exists .$win]} {
        wm withdraw .$win
        wm deiconify .$win
    }
}

itcl::class TimerStack {
    private variable id
    private variable name
    private variable num_elements 0

    constructor {args} {}
    destructor {}
    method push {num project_name} {
        global ::fx_config
        set id($num_elements) $num
        set name($num_elements) $project_name
        incr num_elements
        if {$num_elements == 1} {.menubar.timer.menu add sep}
        .menubar.timer.menu add command -label "$num_elements. Job#: $num - $project_name" \
                -command "[list showStopwatch $num]; [list addToTimerHistory $num "$project_name"]"
        addToTimerHistory $num "$project_name"
    }

    method pop {num} {
        global ::fx_config
        .menubar.timer.menu delete 0 end
        .menubar.timer.menu add command -label "Launch" -underline 0 -command launchStopwatch
        set j 0
        set k $num_elements
        if {$num_elements > 1} {.menubar.timer.menu add sep}
        for {set i 0} {$i < $k} {incr i} {
            if {$num == $id($i)} {
                incr num_elements -1
            } else {
                set id($j) $id($i)
                set name($j) $name($i)
                .menubar.timer.menu add command -label "[expr $j+1]. Job#: $id($j) - $name($j)" \
                        -command [list showStopwatch $id($j)]
                incr j
            }
        }
    }
    method timerIsOn {num} {
        for {set i 0} {$i < $num_elements} {incr i} {
            if {$id($i) == $num} {
                return 1
            }
        }
        return 0
    }
    method timerOn {} {
        set t [menubar cget -tuple]
        set v [qddb_view define $t {}]
        set maxrows [qddb_instance maxnum $v Job]
        for {set i 1} {$i <= $maxrows} {incr i} {
            qddb_instance switch $v Job $i
            set jobnum [qddb_instance getval $v Job.JobNumber]
            if {[timerIsOn $jobnum]} {
                catch [list qddb_view delete $v]
                return 1
            }
        }
        catch [list qddb_view delete $v]
        return 0
    }

    method getNumberOfElements {} {
        return $num_elements
    }
}

#
# try_animate -- 
#
# Called from within animate in an "after" call
# This hack needed because after Dismiss, there is no longer
# a stopwatch object to execute the animate when the "after" method executes
#
proc try_animate { t } {
    catch "$t animate"
}

itcl::class Stopwatch {
    variable jobnum ""
    variable name ""
    variable clientNumber ""
    variable timing 0
    variable startTime ""
    variable startTimeInSeconds ""
    variable endTime ""
    variable endTimeInSeconds ""
    variable duration ""
    variable totalTime ""
    variable totalTimeInSeconds ""
    variable totalSeconds ""
    variable startButton
    variable stopButton
    variable dismissButton

    constructor {args} {
        global ::fx_config ::lwidth ::gv_attr ::gv_fields

        set jobnum $gv_attr(Job.JobNumber)
        set clientNumber $gv_attr(Client.Number)
        set name [string range $this 2 end]
        set w .$name
        toplevel $w
        wm title $w "Stopwatch ($gv_attr(Job.JobNumber): $gv_attr(Job.JobName))"
	wm iconname $w "Watch"
        wm withdraw $w
        set x $w.sw_1
        pack [frame $x] -side top -fill x
        label $x.l1 -text "Job#" -width $lwidth -anchor e
        entry $x.jobnum -textvariable gv_fields([set name]_JobNumber)\
                -width $fx_config(Job.JobNumber,width) \
                -state disabled
        label $x.l2 -text "Job name"
        entry $x.jobname -textvariable gv_fields([set name]_JobName) \
                -state disabled
        pack $x.l1 $x.jobnum $x.l2 -side left 
	pack $x.jobname -side left -expand true -fill x

        set x $w.sw_2
        pack [frame $x] -side top -fill x
        label $x.l1 -text "Job abstract"  -width $lwidth  -anchor e
        entry $x.jobabstract -textvariable gv_fields([set name]_JobAbstract) -state disabled
        pack $x.l1 -side left
	pack $x.jobabstract -side left -expand true -fill x

        set x $w.sw_3
        pack [frame $x] -side top -fill x
        label $x.l1 -text "Start time"  -width $lwidth -anchor e
        entry $x.st -textvariable gv_fields([set name]_WorkUnit.StartTime) \
                -width 20 -state disabled
        label $x.l2 -text "End time"
        entry $x.et -textvariable gv_fields([set name]_WorkUnit.EndTime) \
                -width 20 -state disabled
        pack $x.l1 $x.st $x.l2 $x.et -side left

        set x $w.sw_4
        pack [frame $x] -side top -fill x
        label $x.l3 -text "Duration" -width $lwidth -anchor e
        entry $x.dur -textvariable gv_fields([set name]_WorkUnit.Duration) \
                -width $fx_config(Job.WorkUnit.Duration,width) \
                -state disabled
        label $x.l1 -text "Total time charged"
        entry $x.e -textvariable gv_fields([set name]_TotalTimeCharged) \
                -width $fx_config(Job.TotalTimeCharged,width) \
                -state disabled
        pack $x.l3 $x.dur $x.l1 $x.e -side left

        set x $w.sw_5
        pack [frame $x] -side top -fill x
        label $x.l1 -text "Work unit log"  -width $lwidth  -anchor w
        pack $x.l1 -side top -fill x
	
	set x $w.sw_6
        pack [frame $x] -side top -expand true -fill both
        text $x.t -setgrid true -wrap word -width $fx_config(Job.WorkUnit.Comments,width) \
                -height $fx_config(Job.WorkUnit.Comments,height) \
                -yscrollcommand "$x.s set" \
                -state disabled
        scrollbar $x.s -orient vert -command "$x.t yview"
        pack $x.s -side right -fill y
        pack $x.t -side left -fill both -expand true

        ButtonBox $w.box -orient horizontal
        set startButton [$w.box add -text Start -underline 0 -command [list $this doStartButton] -width 6]
        set stopButton [$w.box add -text Stop -underline 1 -command [list $this doStopButton] \
                -width 6 -state disabled]
        set dismissButton [$w.box add -text Dismiss -underline 1 -command [list $this doDismissButton] \
		-width 6]
        pack $w.box -side bottom -fill x
        set gv_fields([set name]_JobNumber) $gv_attr(Job.JobNumber)
        set gv_fields([set name]_JobName) $gv_attr(Job.JobName)
        set gv_fields([set name]_JobAbstract) $gv_attr(Job.JobAbstract)
        set gv_fields([set name]_TotalTimeCharged) $gv_attr(Job.TotalTimeCharged)
        wm deiconify $w
    }
    destructor {
        timerStack pop $jobnum
        catch "destroy .$name"
    }

    #
    # If main window has the same Client Number as this stopwatch instance
    # then reread the tuple to refresh the data
    #
    method reloadMainWindow {} {
        global ::gv_attr ::gv_fields ::fx:mode_variable

        if {[string compare $clientNumber $gv_attr(Client.Number)] != 0} {
            return
        }
        if {${fx:mode_variable} != "Read-only Mode" && ${fx:mode_variable} != "Change Mode"} {
            return
        }
	set s [menubar cget -schema]
	set k [qddb_search $s -prunebyattr Job.JobNumber $jobnum]
	set tbl [qddb_rows select -format table -print Job.JobNumber -rowdescs on $k]
	qddb_keylist delete $k
	if {$tbl == ""} {
	    fx_dialog .error_dialog "Warning" "Job number $jobnum has been deleted" error 0 Ok
	    deleteFromTimerHistory $jobnum $jobname
	    return
	}
	menubar SearchModeProc
	set maxrow [qddb_table row maxnum $tbl]
	if {$maxrow > 1} {
	    fx_dialog .error_dialog "DATABASE ERROR" \
		    "Job number $jobnum appears $maxrow times in your database!" \
		    error 0 Ok
	    return
	}
	menubar SetLastSearch $tbl
	menubar ChangeModeProc 0
	menubar ReadOnlyModeProc
    }
    method setTotalSeconds {} {
        global ::gv_schema ::gv_fields

        set k [qddb_search $gv_schema -prunebyattr Job.JobNumber $gv_fields([set name]_JobNumber)]
	set t ""
        foreach i [qddb_keylist get $k] {
            set t [qddb_tuple read $gv_schema $i]
            if {[string compare $t ""] == 0} {continue} {break}
        }
        if {[string compare $t ""] == 0} {
            fx_dialog .$name.err_dlg "Please check your database." \
		    "Jobnumber [set name]_JobNumber not found in database!" \
                    error 0 Ok
            return 0
        }
        set v [qddb_view define $t {}]
        set maxrows [qddb_instance maxnum $v Job]
        set found 0
        for {set i 1} {$i <= $maxrows} {incr i} {
            qddb_instance switch $v Job $i
            if {[string compare [qddb_instance getval $v Job.JobNumber] $jobnum] == 0} {
                set found 1
                break;
            }
        }
        if {!$found} {
            fx_dialog .$name.err_dlg "Corrupt Job number in database!" \
                    "Job number $jobnum appears to be corrupt. Fix before proceeding." error 0 Ok
            qddb_keylist delete $k
            qddb_view delete $v
            qddb_tuple delete $t
            return 0
        }
        set tmpTotalTimeCharged [qddb_instance getval $v Job.TotalTimeCharged]
        set totalSeconds [getTotalSeconds $tmpTotalTimeCharged]
        qddb_keylist delete $k
        qddb_view delete $v
        qddb_tuple delete $t
	return 1
    }
    method computeTime {} {
        global ::gv_fields ::fx_config
        if {[setTotalSeconds] == 0} {
	    return 0
	}
	if {$fx_config(datestyle) == "us"} {
	    set endTime [qddb_util formatdate "%m/%d/%Y %H:%M:%S" now]
	} else {
	    set endTime [qddb_util formatdate "%d/%m/%Y %H:%M:%S" now]
	}
        set endTimeInSeconds [qddb_util formatdate "%s" $endTime]
        set duration [expr $endTimeInSeconds - $startTimeInSeconds]
        set totalTimeInSeconds [expr $totalSeconds + $duration]
        set totalTime [formatTime $totalTimeInSeconds]
        set gv_fields([set name]_WorkUnit.EndTime) $endTime
        set gv_fields([set name]_WorkUnit.Duration) [formatTime $duration]
        set gv_fields([set name]_WorkUnit.DurationHours) [getTimeInHours [formatTime $duration]]
        set gv_fields([set name]_TotalTimeCharged) $totalTime
        set gv_fields([set name]_TotalHoursCharged) [getTimeInHours $totalTime]
	return 1
    }
    method saveWorkUnit {} {
        global ::gv_schema ::gv_fields ::fx_config
        set k [qddb_search $gv_schema -prunebyattr Job.JobNumber $gv_fields([set name]_JobNumber)]
        set r [qddb_rows select -format rows -deldup_rows on -attrs {Job.JobNumber} $k]
        set t [lindex [lindex $r 0] 0]
        qddb_keylist delete $k
	if {$t == ""} {
	    return
	}
	while {[qddb_tuple lock $t] == 0} {
	    fx_dialog .$name.err_dlg "Unable to lock client record" \
		    "Unable to lock client record for writing." error 0 Ok
	}
        set v [qddb_view define $t {}]
        qddb_view set $v [lindex [lindex $r 0] 1]
        set tmpTotalTimeCharged [qddb_instance getval $v Job.TotalTimeCharged]
        set totalSeconds [getTotalSeconds $tmpTotalTimeCharged]
	if {$fx_config(datestyle) == "us"} {
	    set endTime [qddb_util formatdate "%m/%d/%Y %H:%M:%S" now]
	} else {
	    set endTime [qddb_util formatdate "%d/%m/%Y %H:%M:%S" now]
	}
        set endTimeInSeconds [qddb_util formatdate "%s" $endTime]
        set duration [expr $endTimeInSeconds - $startTimeInSeconds]
        set totalTimeInSeconds [expr $totalSeconds + $duration]
        set totalTime [formatTime $totalTimeInSeconds]
        set gv_fields([set name]_WorkUnit.EndTime) $endTime
        set gv_fields([set name]_WorkUnit.Duration) [formatTime $duration]
        set gv_fields([set name]_WorkUnit.DurationHours) [getTimeInHours [formatTime $duration]]
        set gv_fields([set name]_TotalTimeCharged) $totalTime
        set gv_fields([set name]_TotalHoursCharged) [getTimeInHours $totalTime]
        set tmpHoursCharged [formatTimeInHours $totalTimeInSeconds]
        set tmpHourlyRate [qddb_instance getval $v Job.HourlyRate]
        set tmpDiscountPercent [qddb_instance getval $v Job.DiscountPercent]
        set update 1
        if {[catch "expr $tmpHoursCharged * $tmpHourlyRate" total] != 0} {
            fx_dialog .$name.err_dlg "Unable to compute amount due!" \
                    "Please ensure hourly rate is a number.  AmountDue not computed." error 0 Ok
            set update 0
        }
        if {$tmpDiscountPercent < 0 || $tmpDiscountPercent > 100} {
            fx_dialog .err_dlg "Discount percent entry error." \
                    "Discount percent must be between 0 and 100." error 0 Ok
            setup update 0
        }

        if {[qddb_instance getval $v Job.WorkUnit.StartDate] != "" || \
                [qddb_instance getval $v Job.WorkUnit.StartTime] != "" || \
                [qddb_instance getval $v Job.WorkUnit.EndDate] != "" || \
                [qddb_instance getval $v Job.WorkUnit.EndTime] != "" || \
                [qddb_instance getval $v Job.WorkUnit.Duration] != "" || \
                [qddb_instance getval $v Job.WorkUnit.Comments] != ""} {
            set new [qddb_instance new $v Job.WorkUnit]
            qddb_instance move $v Job.WorkUnit $new 1
            qddb_instance switch $v Job.WorkUnit 1
        }
        qddb_instance setval $v Job.WorkUnit.StartDate [lindex $startTime 0]
        qddb_instance setval $v Job.WorkUnit.StartTime [lindex $startTime 1]
        qddb_instance setval $v Job.WorkUnit.EndDate [lindex $endTime 0]
        qddb_instance setval $v Job.WorkUnit.EndTime [lindex $endTime 1]
        qddb_instance setval $v Job.WorkUnit.Duration [formatTime $duration]
        qddb_instance setval $v Job.WorkUnit.DurationHours [getTimeInHours [formatTime $duration]]
        qddb_instance setval $v Job.WorkUnit.Comments [.$name.sw_6.t get 0.0 end]

        if {$update} {
            set tmpDiscountAmount [expr $tmpDiscountPercent / 100.0 * $total]
            set fmtDiscount [currency_format $tmpDiscountAmount ROUNDUP]
            set fmtAmount [currency_format $total ROUNDUP]
            set tmpAmountDue [expr $fmtAmount - $fmtDiscount]
            set fmtAmountDue [currency_format $tmpAmountDue ROUNDUP]
            qddb_instance setval $v Job.TotalAmountDue $fmtAmountDue
            qddb_instance setval $v Job.DiscountAmount $fmtDiscount
            qddb_instance setval $v Job.TotalTimeCharged $totalTime
            qddb_instance setval $v Job.TotalHoursCharged [getTimeInHours $totalTime]
            qddb_instance setval $v Job.EndDate [lindex $endTime 0]
        }

        update idletasks; update
        qddb_tuple write $t
        qddb_tuple delete $t
        reloadMainWindow
    }
    method doStopButton {} {
        global ::gv_fields
        set timing 0
        $stopButton configure -state disabled
        .$name.sw_6.t configure -state disabled
        saveWorkUnit
        $startButton configure -state normal
        $dismissButton configure -state normal
        focus $startButton
    }
    method animate {} {
        global ::fx_config
        if {$timing == 0} {return}
	computeTime
        after [expr $fx_config(\$usersetup\$,\$splitrefresh\$) * 1000] [list try_animate $this]
    }
    method doStartButton {} {
        global ::gv_fields ::fx_config
        set timing 1
        $startButton configure -state disabled
        $dismissButton configure -state disabled
        set timing 1
	if {$fx_config(datestyle) == "us"} {
	    set startTime [qddb_util formatdate "%m/%d/%Y %H:%M:%S" now]
	} else {
	    set startTime [qddb_util formatdate "%d/%m/%Y %H:%M:%S" now]
	}
        set startTimeInSeconds [qddb_util formatdate "%s" $startTime]
        set gv_fields([set name]_WorkUnit.StartTime) $startTime
        set gv_fields([set name]_WorkUnit.EndTime) ""
        set gv_fields([set name]_WorkUnit.Duration) ""
        $stopButton configure -state normal
        .$name.sw_6.t configure -state normal
        .$name.sw_6.t delete 0.0 end
        focus $stopButton
        animate
    }
    method doDismissButton {} {
        itcl::delete object $this
    }
}
