# 
# Copyright (C) 1997-2001 Herrin Software Development, Inc.
# All rights reserved.
#
# This file is part of Qtime
#

#
# currency_format { amt how }
#     amt ...... must be a valid number or bad things will happen
#     how ...... TRUNCATE, ROUNDUP, or ANYTHINGELSE
proc currency_format { amt how {fmt %8.2f} } {
    set rv [format $fmt $amt]
    set dot [string first . $rv]
    set a [string range $rv 0 [expr $dot+1]]
    set b [string index $rv [expr $dot+2]]
    set c [string index $rv [expr $dot+3]]
    if { $how == "TRUNCATE" } {
        return [string trim $a$b " "]
    } elseif { $how == "ROUNDUP" } {
        if { $c == 0 || $c == ""} { 
            return [string trim $a$b " "]
        } else { 
            incr b 
            if { $b == 0 } { incr a }
            return [string trim $a$b " "]
        }
    } else { 
        if { $c > 5 } { 
            incr b 
            if { $b == 0 } { incr a }
            return [string trim $a$b " "]
        }
        return [string trim $a$b " "]
    }
}

proc isnumber { n } {
    if {[catch [list expr double($n)]] != 0} {
        return 0
    } else {
        return 1
    }
}

proc formatTimeInHours {seconds} {
    return [expr $seconds.0 / 3600.0]
}

proc formatTime {seconds} {
    set h [expr $seconds / 3600]
    set m [expr ($seconds - ($h * 3600)) / 60]
    set s [expr ($seconds - ($h * 3600) - ($m * 60))]
    set rv [format "%d:%02d:%02d" $h $m $s]
    return $rv
}

proc parseTime { s p } {
    set len [string length $s]
    switch -exact $p {
        "h" {set rv [string range $s 0 [expr $len - 7]]}
        "m" {set rv [string range $s [expr $len - 5] [expr $len - 4]]}
        "s" {set rv [string range $s [expr $len - 2] end]}
        default {set rv 0}
    }
    if {$rv == ""} { set rv 0 }
    return $rv
}

proc getTotalSeconds {totalTime} {
    if {$totalTime == ""} {
	return 0
    }
    set seconds [parseTime $totalTime s]
    if {[string range $seconds 0 0] == "0"} {set seconds [string range $seconds 1 1]}
    set seconds [format "%d" $seconds]
    set minutes [parseTime $totalTime m]
    if {[string range $minutes 0 0] == "0"} {set minutes [string range $minutes 1 1]}
    set minutes [format "%d" $minutes]
    set hours [parseTime $totalTime h]
    set hours [format "%d" $hours]
    return [expr ($hours * 3600) + ($minutes * 60) + $seconds]
}

proc getDuration {startDate startTime endDate endTime} {
    if {[qddb_util isdate "$startDate $startTime"] == 0 || \
            [qddb_util isdate "$endDate $endTime"] == 0} {
        return -1
    }
    set begin [qddb_util formatdate %s "$startDate $startTime"]
    set end [qddb_util formatdate %s "$endDate $endTime"]
    return [expr $end - $begin]
}

proc getTimeInHours {totalTime} {
    set totalSeconds [getTotalSeconds $totalTime]
    return [formatTimeInHours $totalSeconds]
}

proc Qtime:UserSetup {} {
    global fx_setup_config fx_config tmp_usersetup tmp_menu

    set setup_lwidth 10
    set setup_ewidth 40
    set r .user_setup
    if {[winfo exists $r]} {
        raise $r
        focus $r
        return
    }
    toplevel $r
    wm withdraw $r
    wm title $r "User Setup"

    set tmp_usersetup(company) "$fx_config(\$usersetup\$,\$company\$)"
    set tmp_usersetup(street) "$fx_config(\$usersetup\$,\$street\$)"
    set tmp_usersetup(city) "$fx_config(\$usersetup\$,\$city\$)"
    set tmp_usersetup(state) "$fx_config(\$usersetup\$,\$state\$)"
    set tmp_usersetup(zip) "$fx_config(\$usersetup\$,\$zip\$)"
    set tmp_usersetup(phone) "$fx_config(\$usersetup\$,\$phone\$)"
    set tmp_usersetup(email) "$fx_config(\$usersetup\$,\$email\$)"
    set tmp_usersetup(fax) "$fx_config(\$usersetup\$,\$fax\$)"
    set tmp_usersetup(web) "$fx_config(\$usersetup\$,\$web\$)"
    set tmp_usersetup(cc) "$fx_config(\$usersetup\$,\$cc\$)"
    set tmp_usersetup(splitrefresh) "$fx_config(\$usersetup\$,\$splitrefresh\$)"
    set tmp_usersetup(reportseconds) "$fx_config(\$usersetup\$,\$reportseconds\$)"

    set w $r.nb
    NoteBook $r.nb
    $w insert end instructions -text "Instructions"
    $w insert end address -text "Address/Misc"
    $w insert end preferences -text "Preferences"
    $w raise instructions

    set f [$w getframe instructions].f1
    pack [frame $f] -side top -expand on -fill both
    label $f.l -bd 0 -relief flat -justify left -text {
This "User Setup" window allows you to
define your company name, address email
and so forth.  To set these options, click
the "Address/Misc" notebook panel and
fill-in the appropriate values in the
entries.  You may also choose among
several options to tell Qtime how to
behave.  To setup these configurable
options, click the "Preferences" notebook
panel.

To save your settings after you leave the
current session, you must be sure to
choose "Configure/Personal configuration/Save".
If you do not save these options, your
changes will not be remembered the next time
you invoke Qtime.
}
    pack $f.l -side top -fill both
    
    set f [$w getframe address].f2
    pack [frame $f] -side top -expand on -fill both

    set lf $f.f1
    pack [frame $lf] -side top -expand on -fill x
    pack [label $lf.l -text "Company" -width $setup_lwidth -anchor e] -side left
    pack [entry $lf.e -textvariable tmp_usersetup(company) -width $setup_ewidth] -side left
    set lf $f.f2
    pack [frame $lf] -side top -expand on -fill x
    pack [label $lf.l -text "Street" -width $setup_lwidth -anchor e] -side left
    pack [entry $lf.e -textvariable tmp_usersetup(street) -width $setup_ewidth] -side left
    set lf $f.f3
    pack [frame $lf] -side top -expand on -fill x
    pack [label $lf.l -text "City" -width $setup_lwidth -anchor e] -side left
    pack [entry $lf.e -textvariable tmp_usersetup(city) -width $setup_ewidth] -side left
    set lf $f.f4
    pack [frame $lf] -side top -expand on -fill x
    pack [label $lf.l -text "State" -width $setup_lwidth -anchor e] -side left
    pack [entry $lf.e -textvariable tmp_usersetup(state) -width $setup_ewidth] -side left
    set lf $f.f5
    pack [frame $lf] -side top -expand on -fill x
    pack [label $lf.l -text "Zip" -width $setup_lwidth -anchor e] -side left
    pack [entry $lf.e -textvariable tmp_usersetup(zip) -width $setup_ewidth] -side left
    set lf $f.f6
    pack [frame $lf] -side top -expand on -fill x
    pack [label $lf.l -text "Phone" -width $setup_lwidth -anchor e] -side left
    pack [entry $lf.e -textvariable tmp_usersetup(phone) -width $setup_ewidth] -side left
    set lf $f.f7
    pack [frame $lf] -side top -expand on -fill x
    pack [label $lf.l -text "Email" -width $setup_lwidth -anchor e] -side left
    pack [entry $lf.e -textvariable tmp_usersetup(email) -width $setup_ewidth] -side left
    set lf $f.f8
    pack [frame $lf] -side top -expand on -fill x
    pack [label $lf.l -text "Fax" -width $setup_lwidth -anchor e] -side left
    pack [entry $lf.e -textvariable tmp_usersetup(fax) -width $setup_ewidth] -side left
    set lf $f.f9
    pack [frame $lf] -side top -expand on -fill x
    pack [label $lf.l -text "Web page" -width $setup_lwidth -anchor e] -side left
    pack [entry $lf.e -textvariable tmp_usersetup(web) -width $setup_ewidth] -side left
    set lf $f.f10
    pack [frame $lf] -side top -expand on -fill x
    pack [label $lf.l -text "cc" -width $setup_lwidth -anchor e] -side left
    pack [entry $lf.e -textvariable tmp_usersetup(cc) -width $setup_ewidth] -side left

    set f [$w getframe preferences].f1
    pack [frame $f -bd 3 -relief groove] -side top -expand on -fill both
    set lf $f.f1
    pack [frame $lf] -side top -fill x
    SpinBox $lf.a -label "Split refresh(seconds): " \
            -range {5 60 1} -labelanchor e -labelwidth 25 -width 2 \
	    -modifycmd "set tmp_usersetup(splitrefresh) \[$lf.a getvalue\]"
    pack $lf.a -side top -anchor w

    set lf $f.f2
    pack [frame $lf] -side top -fill x
    ComboBox $lf.a -label "Report seconds: " \
	-textvariable tmp_menu \
	-labelwidth 25 -labelanchor e \
	-width 8 -values {Yes Truncate Round Roundup}
    pack $lf.a -side top -anchor w

    set tmp_menu $tmp_usersetup(reportseconds)
    pack $w -side top -expand on -fill both

    set f $r.bot
    pack [frame $f] -side bottom -fill x
    button $f.ok -text Ok -command [list set fx_setup_config 1]
    pack $f.ok -side left -padx 5m -pady 2m
    button $f.cancel -text Cancel -command [list set fx_setup_config 0]
    pack $f.cancel -side right -padx 5m -pady 2m
    update idletasks

    wm minsize $r [winfo reqwidth $r] [winfo reqheight $r]
    wm maxsize $r [winfo reqwidth $r] [winfo reqheight $r]
    Fx:SetGeometry $r
    wm deiconify $r
    wm protocol $r WM_DELETE_WINDOW {set fx_setup_config 0}
    tkwait variable fx_setup_config
    Fx:SavePosition $r
    if {$fx_setup_config} {
        set fx_config(\$usersetup\$,\$company\$) "$tmp_usersetup(company)"
        set fx_config(\$usersetup\$,\$street\$) "$tmp_usersetup(street)"
        set fx_config(\$usersetup\$,\$city\$) "$tmp_usersetup(city)"
        set fx_config(\$usersetup\$,\$state\$) "$tmp_usersetup(state)"
        set fx_config(\$usersetup\$,\$zip\$) "$tmp_usersetup(zip)"
        set fx_config(\$usersetup\$,\$phone\$) "$tmp_usersetup(phone)"
        set fx_config(\$usersetup\$,\$email\$) "$tmp_usersetup(email)"
        set fx_config(\$usersetup\$,\$fax\$) "$tmp_usersetup(fax)"
        set fx_config(\$usersetup\$,\$web\$) "$tmp_usersetup(web)"
        set fx_config(\$usersetup\$,\$cc\$) "$tmp_usersetup(cc)"
        set fx_config(\$usersetup\$,\$splitrefresh\$) "$tmp_usersetup(splitrefresh)"
        set fx_config(\$usersetup\$,\$reportseconds\$) "$tmp_menu"
    }
    destroy $r
}

proc Qtime:Stabilize {} {
    global fx:status_variable gv_schema

    set cancel [fx_dialog .dialog "Continue?" "Do you really want to stabilize the database?" \
            warning 0 Ok Cancel]
    if {$cancel} {
        set fx:status_variable "Stabilization cancelled."
        return
    }
    set schema [menubar cget -schema]
    set path [qddb_schema path $schema]
    menubar KillLastSearch
    qddb_schema close $schema
    update idletasks; update
    fx_message .msg "Stabilizing, please wait..."
    set fx:status_variable "Stabilizing..."
    ::blt::busy hold .
    update idletasks; update
    set failure [catch "exec qstall $path"]
    catch "destroy .msg"
    if {$failure} {
        fx_dialog .dialog "Stabilization failure" "Stabilization failed!  Someone may \
                be using this database." error 0 Ok
        set fx:status_variable "Stabilization failed!"
    } else {
        fx_dialog .dialog "Stabilization success" "Stabilization succeeded." info 0 Ok
        set fx:status_variable "Stabilization succeeded!"
    }
    set schema [qddb_schema open $path]
    menubar configure -schema $schema
    =Job configure -setschema $schema
    set gv_schema $schema
    ::blt::busy release .
}

proc Qtime:AboutBox {} {
    global qddb_library qtime_version fx_config

    if {[winfo exists .aboutBox]} {
        wm withdraw .aboutBox
        wm deiconify .aboutBox
        focus .aboutBox
        update idletasks
        return
    }
    toplevel .aboutBox
    wm title .aboutBox "About Qtime"
    if {[info exists fx_config(geom,\$AboutBox\$)]} {
        set x [split $fx_config(geom,\$AboutBox\$) +]
        set y [lindex $x 2]
        set x [lindex $x 1]
        wm geometry .aboutBox +$x+$y
    }
    set x .aboutBox.buttons
    frame $x
    pack $x -side bottom -fill x
    button $x.ok -text "Ok" -command {
        global fx_config
        set fx_config(geom,\$AboutBox\$) [wm geometry .aboutBox]
        destroy .aboutBox
    }
    pack $x.ok -fill x

    message .aboutBox.version -justify center -aspect 400 \
            -text "Qddb Time Tracking Database\nVersion ${qtime_version}\n"
    pack .aboutBox.version -side top

    message .aboutBox.copyright -justify center -aspect 500 \
            -text "Copyright (c) 1997-2001\nHerrin Software Development, Inc.\nAll rights reserved.\n" 
    pack .aboutBox.copyright -side top

    message .aboutBox.license -justify left -aspect 250 \
            -text "This is Qtime, the Qddb Time Tracking Database.

Copyright (C) 1993-2001 Herrin Software Development, Inc.
All rights reserved.
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    "
    pack .aboutBox.license -side top
    message .aboutBox.subscription -justify left -aspect 1000 \
            -text "See \"http://www.hsdi.com/qddb\" for the latest release, \
            support, and subscription information."
    pack .aboutBox.subscription -side top
}

proc Qtime:ManPage {} {
    global qtime_library qtime_version fx_config

    if {[winfo exists .manPage]} {
        wm withdraw .manPage
        wm deiconify .manPage
        focus .manPage
        update idletasks
        return
    }
    toplevel .manPage
    wm title .manPage "qtime manual page"
    set x .manPage.buttons
    frame $x -highlightthickness 0
    pack $x -side bottom -fill x
    button $x.ok -text "Ok" -command "
    set [list fx_config(geom,\$manPage\$)] \[wm geometry .manPage\]
    destroy .manPage
    "
    pack $x.ok -fill x

    message .manPage.label -justify center -aspect 1000 \
            -text "qtime(1) Manual Page"
    pack .manPage.label -side top

    message .manPage.version -justify center -aspect 1000 \
            -text "Qddb Time Tracking Database\nVersion $qtime_version\n"
    pack .manPage.version -side top

    message .manPage.copyright -justify center -aspect 1000 \
            -text "Copyright (c) 1997-2001\nHerrin Software Development, Inc.\nAll rights reserved.\n" 
    pack .manPage.copyright -side top

    set fd [open $qtime_library/xqtime.0 "r"]
    set MAN [read $fd]
    close $fd
    set x .manPage.f
    frame $x -highlightthickness 0
    pack $x -side top -expand on -fill both
    scrollbar $x.s -orient vertical -relief sunken -command [list $x.c yview]
    pack $x.s -side right -fill y
    text $x.c -relief sunken -bd 2 -yscrollcommand [list $x.s set] -wrap word -setgrid on
    pack $x.c -side left -expand on -fill both
    $x.c insert 0.0 $MAN
    $x.c configure -state disabled
    if {[info exists fx_config(geom,\$manPage\$)]} {
        wm geometry .manPage $fx_config(geom,\$manPage\$)
    } else {
        wm geometry .manPage +70+0
    }
}

proc doRoundingAdjustment {} {
    global gv_attr fx_config

    set r $fx_config(\$usersetup\$,\$reportseconds\$)
    set s [parseTime $gv_attr(Job.TotalTimeCharged) s]
    if {$r == "Yes" || $s == "00"} {return}

    set v [menubar cget -view]
    set new [qddb_instance new $v Job.WorkUnit]
    qddb_instance move $v Job.WorkUnit $new 1
    qddb_instance switch $v Job.WorkUnit 1
    qddb_instance setval $v Job.WorkUnit.StartDate "today"
    qddb_instance setval $v Job.WorkUnit.EndDate "today"
    qddb_instance setval $v Job.WorkUnit.Comments "SECONDS ADJUSTMENT"

    if {$r == "Truncate"} {
	qddb_instance setval $v Job.WorkUnit.StartTime "23:59:59"
	qddb_instance setval $v Job.WorkUnit.EndTime "23:59:[expr 59 - $s]"
    } elseif {$r == "Roundup"}  {
	qddb_instance setval $v Job.WorkUnit.StartTime "00:00:00"
	qddb_instance setval $v Job.WorkUnit.EndTime "00:00:[expr 60 - $s]"
    } elseif {$s < 30} {    ;# ROUND DOWN
	qddb_instance setval $v Job.WorkUnit.StartTime "23:59:59"
	qddb_instance setval $v Job.WorkUnit.EndTime "23:59:[expr 59 - $s]"
    } else {                ;# ROUND UP
	qddb_instance setval $v Job.WorkUnit.StartTime "00:00:00"
	qddb_instance setval $v Job.WorkUnit.EndTime "00:00:[expr 60 - $s]"
    }

    doRecompute
}

proc getStatementText {} {
    global gv_attr fx_config

    doRoundingAdjustment

    set r $fx_config(\$usersetup\$,\$reportseconds\$)

    set output ""
    set hourlyRate [format %10.2f  $gv_attr(Job.HourlyRate)]
    set subtotal [format %10.2f  [expr $gv_attr(Job.DiscountAmount) + $gv_attr(Job.TotalAmountDue)]]
    set discountPercent [format %10.2f  $gv_attr(Job.DiscountPercent)]
    set discountAmount [format %10.2f  $gv_attr(Job.DiscountAmount)]
    set totalDue [format %10.2f  $gv_attr(Job.TotalAmountDue)]
    
    lappend output "                          $fx_config(\$usersetup\$,\$company\$)"
    lappend output "                          $fx_config(\$usersetup\$,\$street\$)"
    lappend output "                          $fx_config(\$usersetup\$,\$city\$), $fx_config(\$usersetup\$,\$state\$)  $fx_config(\$usersetup\$,\$zip\$)"
    lappend output "                          Voice: $fx_config(\$usersetup\$,\$phone\$)"
    lappend output "                          Email: $fx_config(\$usersetup\$,\$email\$)"
    lappend output "                          Fax:   $fx_config(\$usersetup\$,\$fax\$)"
    lappend output "                          WWW:   $fx_config(\$usersetup\$,\$web\$)"
    lappend output ""
	if {$gv_attr(Client.Company) != "" && \
			$gv_attr(Client.Address.Street) != "" && \
			$gv_attr(Client.Address.City) != "" && \
			$gv_attr(Client.Address.State) != "" && \
			$gv_attr(Client.Address.Zip) != ""} {
		lappend output "$gv_attr(Client.Company)"
		lappend output "$gv_attr(Client.Address.Street)"
		lappend output \
				"$gv_attr(Client.Address.City), $gv_attr(Client.Address.State)  $gv_attr(Client.Address.Zip)"
	} else {
        fx_dialog .dialog "Empty name/address field(s)" \
				"One or more name and/or address field(s) is empty, omitting them from report." \
				info 0 Ok
	}
    lappend output ""
    lappend output "Contact:      $gv_attr(Client.Name)"
    lappend output "Client#:      $gv_attr(Client.Number)"
    lappend output "Job#:         $gv_attr(Job.JobNumber)"
    lappend output "Job name:     $gv_attr(Job.JobName)"
    lappend output "Job abstract: $gv_attr(Job.JobAbstract)"
    lappend output ""
    lappend output "Work Summary"
    lappend output "------------"
    lappend output "Start date:                $gv_attr(Job.StartDate)"
    lappend output "End date:                  $gv_attr(Job.EndDate)"

    if {$r == "Yes"} {
	lappend output "Total time charged(H:M):   $gv_attr(Job.TotalTimeCharged)"
    } else {
	set time [string range $gv_attr(Job.TotalTimeCharged) 0 \
		[expr [string length $gv_attr(Job.TotalTimeCharged)] - 4]]
	lappend output "Total time charged(H:M):   $time"
    }
    lappend output "Work classification:       $gv_attr(Job.WorkClassification)"
    lappend output "Resources used:            $gv_attr(Job.ResourcesUsed)"
    lappend output "Work performed by:         $gv_attr(Job.WorkPerformedBy)"
    lappend output "Hourly rate:             $hourlyRate"
    lappend output "Subtotal:                $subtotal"
    lappend output "Discount %:              $discountPercent"
    lappend output "DiscountAmount:          $discountAmount"
    lappend output "Total due:               $totalDue"
    lappend output ""
    lappend output "Job comments"
    lappend output "------------"
    lappend output "$gv_attr(Job.Comments)"
}

proc doStatementPrint {} {
    global fx:status_variable fx_config gv_attr fx

    set fx ""
    Fx_PrintDialog qtime_print_dialog -toplevel "" -title "Print statement"
    if {$fx_config(cancel_print) == 0} {
        blt::busy hold .
        if {[winfo exists .search_results]} {
            blt::busy hold .search_results
        }
        update idletasks
        set output [getStatementText]
        Fx:Print $output
        catch "blt::busy forget ."
    } else {
        set $fx:status_variable "Printing cancelled."
    }
}

proc doStatementEmail {} {
    global gv_attr to cc bcc sub body gv_user_response tmp_setup fx_config

    set label_width 10
    set e .email
    set gv_user_response 0
    toplevel $e
    wm withdraw $e
    wm title $e "E-mail Statement"

    pack [frame $e.f1] -side top -fill x
    label $e.f1.l -width $label_width -text "To:" -anchor e
    entry $e.f1.e -textvariable to
    pack $e.f1.l -side left
    pack $e.f1.e -side left -fill x -expand yes

    pack [frame $e.f2] -side top -fill x
    label $e.f2.l -width $label_width -text "cc:" -anchor e
    entry $e.f2.e -textvariable cc
    pack $e.f2.l -side left 
    pack $e.f2.e -side left -fill x -expand yes

    pack [frame $e.f2_1] -side top -fill x
    label $e.f2_1.l -width $label_width -text "bcc:" -anchor e
    entry $e.f2_1.e -textvariable bcc
    pack $e.f2_1.l -side left 
    pack $e.f2_1.e -side left -fill x -expand yes

    pack [frame $e.f3] -side top -fill x
    label $e.f3.l -width $label_width -text "Subject:" -anchor e
    entry $e.f3.e -textvariable sub 
    pack $e.f3.l -side left
    pack $e.f3.e -side left -fill x -expand yes

    set w $e.f4
    pack [frame $w] -side top -fill both -expand yes
    label $w.l -text "Message:" -anchor w
    pack $w.l -side top -fill x
    scrollbar $w.s -relief sunken -command [list $w.t yview]
    text $w.t -relief sunken -bd 2 -yscroll [list $w.s set]
    pack $w.s -side right -fill y
    pack $w.t -side left -fill both -expand yes

    set b $e.buttons
    pack [frame $b -relief groove -bd 2] -side bottom -fill x
    button $b.ok -text "Send message" -command {set gv_user_response "Ok"}
    pack $b.ok -side left -padx 8m -pady 2m
    button $b.cancel -text "Cancel message" -command {set gv_user_response "Cancel"}
    pack $b.cancel -side right -padx 8m -pady 2m

    set to $gv_attr(Client.Email)
    set cc "$fx_config(\$usersetup\$,\$cc\$)"
    set sub "Charges for job#: $gv_attr(Job.JobNumber)"

    set t [getStatementText]
    foreach i $t {
        $w.t insert end "\n$i"
    }
    wm deiconify $e
    set oldGrab [grab current $e]
    catch "grab $e"
    tkwait variable gv_user_response
    while {1} {
	global fx_emailsetup

        if {$gv_user_response == "Ok"} {
	    set email_msg [$w.t get 0.0 end]
	    set email_subj $sub
	    set email_recipient $to
	    if {[string length [string trim $email_recipient]] == 0} {
		continue
	    }
	    set cmdline [list |$fx_emailsetup(program) $fx_emailsetup(subject) "$email_subj"]
	    if {[string length $cc] > 0 && [string length $fx_emailsetup(cc)] > 0} {
		lappend cmdline $fx_emailsetup(cc)
		lappend cmdline $cc
	    }
	    if {[string length $bcc] > 0 && [string length $fx_emailsetup(bcc)] > 0} {
		lappend cmdline $fx_emailsetup(bcc)
		lappend cmdline $bcc
	    }
	    lappend cmdline $email_recipient
	    lappend cmdline ">>&" "/dev/null"
	    if {[catch {
		set fd [open $cmdline w]
		puts $fd "$email_msg"
		close $fd
	    } error] != 0} {
		if {[fx_dialog .dialog "Mail error" "$error" error 0 "Try again" Cancel] == 1} {
		    return
		} else {
		    continue
		}
	    }
            break;
        } elseif {$gv_user_response == "Cancel"} {
            break;
        }
    }
    destroy $e
    catch "grab $oldGrab"
}
